///@file subfunc.cpp - IP and FILE I/O functions.

/** scan (nnn.nnn.nnn.nnn) or [nnn.nnn.nnn.nnn] at line end. */
bool scan_ip(const char* s, unsigned ip[4])
{
	char* p1 = strrchr(s, '(');
	char* p2 = strrchr(s, '[');
	char sep;

	if (p1 > p2 && sscanf(p1, "(%u.%u.%u.%u%c", ip, ip+1, ip+2, ip+3, &sep) == 5 && sep == ')'
	         && ip[0] < 256 && ip[1] < 256 && ip[2] < 256 && ip[3] < 256)
		return true;
	else if (p2 && sscanf(p2, "[%u.%u.%u.%u%c", ip, ip+1, ip+2, ip+3, &sep) == 5 && sep == ']'
	         && ip[0] < 256 && ip[1] < 256 && ip[2] < 256 && ip[3] < 256)
		return true;
	else
		return false;	// cannot find IP numbers.
}

/** judgement privateIP. */
bool is_private_ip(const unsigned ip[4])
{
	// class A privateIP [10.*.*.*/8]
	if (ip[0] == 10)
		return true;

	// class B privateIP [172.16~31.*.*/16]
	if (ip[0] == 172 && (ip[1] >= 16 && ip[1] <= 31))
		return true;

	// class C privateIP [192.168.0~255.*/24]
	if (ip[0] == 192 && ip[2] == 168 && ip[3] <= 255)
		return true;

	// not privateIP
	return false;
}

/** make net address part. */
const char* net_address(const unsigned ip[4])
{
	static char buf[10*4+20];
	if (ip[0] < 128) // class A
		sprintf(buf, "%u.0.0.0/8", ip[0]);
	else if (ip[0] < 192) // class B
		sprintf(buf, "%u.%u.0.0/16", ip[0], ip[1]);
	else if (ip[0] < 224) // class C
		sprintf(buf, "%u.%u.%u.0/24", ip[0], ip[1], ip[2]);
	else if (ip[0] < 240) // class D
		sprintf(buf, "%u.%u.%u.%u(classD)", ip[0], ip[1], ip[2], ip[3]);
	else // class E
		sprintf(buf, "%u.%u.%u.%u(classE)", ip[0], ip[1], ip[2], ip[3]);

	return buf;
}

/** peek a next input char */
int fpeek(FILE* f)
{
	int c = getc(f);
	ungetc(c, f);
	return c;
}

/** getline and append to buf. */
char* fcatgets(char buf[], size_t bufsize, FILE* f)
{
	size_t len = strlen(buf);
	if (bufsize > len)
		return fgets(buf + len, bufsize - len, f);
	else
		return NULL;
}
// subfunc.cpp - end.
